@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "MagickCore/studio.h"
+#include "MagickCore/artifact.h"
 #include "MagickCore/attribute.h"
 #include "MagickCore/cache.h"
 #include "MagickCore/color.h"
@@ -86,33 +87,6 @@
 #  include <libxml/tree.h>
 #endif
 
-/*
-  Definitions
-*/
-#define LCMSHDRI
-#if !defined(MAGICKCORE_HDRI_SUPPORT)
-  #if (MAGICKCORE_QUANTUM_DEPTH == 8)
-  #undef LCMSHDRI
-  #define LCMSScaleSource(pixel)  ScaleQuantumToShort(pixel)
-  #define LCMSScaleTarget(pixel)  ScaleShortToQuantum(pixel)
-  typedef unsigned short
-    LCMSType;
-  #elif (MAGICKCORE_QUANTUM_DEPTH == 16)
-  #undef LCMSHDRI
-  #define LCMSScaleSource(pixel)  (pixel)
-  #define LCMSScaleTarget(pixel)  (pixel)
-  typedef unsigned short
-    LCMSType;
-  #endif
-#endif
-
-#if defined(LCMSHDRI)
-#define LCMSScaleSource(pixel)  (source_scale*QuantumScale*(pixel))
-#define LCMSScaleTarget(pixel) ClampToQuantum(target_scale*QuantumRange*(pixel))
-typedef double
-  LCMSType;
-#endif
-
 /*
   Forward declarations
 */
@@ -375,6 +349,31 @@ MagickExport char *GetNextImageProfile(const Image *image)
 
 #if defined(MAGICKCORE_LCMS_DELEGATE)
 
+typedef struct _LCMSInfo
+{
+  ColorspaceType
+    colorspace;
+
+  cmsUInt32Number
+    type;
+
+  size_t
+    channels;
+
+  cmsHPROFILE
+    profile;
+
+  int
+    intent;
+
+  double
+    scale,
+    translate;
+
+  void
+    **magick_restrict pixels;
+} LCMSInfo;
+
 #if LCMS_VERSION < 2060
 static void* cmsGetContextUserData(cmsContext ContextID)
 {
@@ -400,42 +399,47 @@ static void cmsDeleteContext(cmsContext magick_unused(ContextID))
 }
 #endif
 
-static LCMSType **DestroyPixelThreadSet(LCMSType **pixels)
+static void **DestroyPixelThreadSet(void **pixels)
 {
   register ssize_t
     i;
 
-  if (pixels == (LCMSType **) NULL)
-    return((LCMSType **) NULL);
+  if (pixels == (void **) NULL)
+    return((void **) NULL);
   for (i=0; i < (ssize_t) GetMagickResourceLimit(ThreadResource); i++)
-    if (pixels[i] != (LCMSType *) NULL)
-      pixels[i]=(LCMSType *) RelinquishMagickMemory(pixels[i]);
-  pixels=(LCMSType **) RelinquishMagickMemory(pixels);
+    if (pixels[i] != (void *) NULL)
+      pixels[i]=RelinquishMagickMemory(pixels[i]);
+  pixels=(void **) RelinquishMagickMemory(pixels);
   return(pixels);
 }
 
-static LCMSType **AcquirePixelThreadSet(const size_t columns,
-  const size_t channels)
+static void **AcquirePixelThreadSet(const size_t columns,
+  const size_t channels,MagickBooleanType highres)
 {
-  LCMSType
-    **pixels;
-
   register ssize_t
     i;
 
   size_t
     number_threads;
 
+  size_t
+    size;
+
+  void
+    **pixels;
+
   number_threads=(size_t) GetMagickResourceLimit(ThreadResource);
-  pixels=(LCMSType **) AcquireQuantumMemory(number_threads,sizeof(*pixels));
-  if (pixels == (LCMSType **) NULL)
-    return((LCMSType **) NULL);
+  pixels=(void **) AcquireQuantumMemory(number_threads,sizeof(*pixels));
+  if (pixels == (void **) NULL)
+    return((void **) NULL);
   (void) memset(pixels,0,number_threads*sizeof(*pixels));
+  size=sizeof(double);
+  if (highres == MagickFalse)
+    size=sizeof(Quantum);
   for (i=0; i < (ssize_t) number_threads; i++)
   {
-    pixels[i]=(LCMSType *) AcquireQuantumMemory(columns,channels*
-      sizeof(**pixels));
-    if (pixels[i] == (LCMSType *) NULL)
+    pixels[i]=AcquireQuantumMemory(columns,channels*size);
+    if (pixels[i] == (void *) NULL)
       return(DestroyPixelThreadSet(pixels));
   }
   return(pixels);
@@ -454,10 +458,9 @@ static cmsHTRANSFORM *DestroyTransformThreadSet(cmsHTRANSFORM *transform)
   return(transform);
 }
 
-static cmsHTRANSFORM *AcquireTransformThreadSet(
-  const cmsHPROFILE source_profile,const cmsUInt32Number source_type,
-  const cmsHPROFILE target_profile,const cmsUInt32Number target_type,
-  const int intent,const cmsUInt32Number flags,cmsContext cms_context)
+static cmsHTRANSFORM *AcquireTransformThreadSet(const LCMSInfo *source_info,
+  const LCMSInfo *target_info,const cmsUInt32Number flags,
+  cmsContext cms_context)
 {
   cmsHTRANSFORM
     *transform;
@@ -476,8 +479,9 @@ static cmsHTRANSFORM *AcquireTransformThreadSet(
   (void) memset(transform,0,number_threads*sizeof(*transform));
   for (i=0; i < (ssize_t) number_threads; i++)
   {
-    transform[i]=cmsCreateTransformTHR(cms_context,source_profile,source_type,
-      target_profile,target_type,intent,flags);
+    transform[i]=cmsCreateTransformTHR(cms_context,source_info->profile,
+      source_info->type,target_info->profile,target_info->type,
+      target_info->intent,flags);
     if (transform[i] == (cmsHTRANSFORM) NULL)
       return(DestroyTransformThreadSet(transform));
   }
@@ -516,6 +520,105 @@ static void CMSExceptionHandler(cmsContext context,cmsUInt32Number severity,
     "UnableToTransformColorspace","`%s', %s (#%u)",image->filename,
     message != (char *) NULL ? message : "no message",severity);
 }
+
+static void TransformDoublePixels(const int id,const Image* image,
+  const LCMSInfo *source_info,const LCMSInfo *target_info,
+  const cmsHTRANSFORM *transform,Quantum *q)
+{
+#define GetLCMSPixel(source_info,pixel) \
+  (source_info->scale*QuantumScale*(pixel)+source_info->translate)
+#define SetLCMSPixel(target_info,pixel) \
+  ClampToQuantum(target_info->scale*QuantumRange*(pixel)+target_info->translate)
+
+  register double
+    *p;
+
+  register ssize_t
+    x;
+
+  p=(double *) source_info->pixels[id];
+  for (x=0; x < (ssize_t) image->columns; x++)
+  {
+    *p++=GetLCMSPixel(source_info,GetPixelRed(image,q));
+    if (source_info->channels > 1)
+      {
+        *p++=GetLCMSPixel(source_info,GetPixelGreen(image,q));
+        *p++=GetLCMSPixel(source_info,GetPixelBlue(image,q));
+      }
+    if (source_info->channels > 3)
+      *p++=GetLCMSPixel(source_info,GetPixelBlack(image,q));
+    q+=GetPixelChannels(image);
+  }
+  cmsDoTransform(transform[id],source_info->pixels[id],
+    target_info->pixels[id],(unsigned int) image->columns);
+  p=(double *) target_info->pixels[id];
+  q-=GetPixelChannels(image)*image->columns;
+  for (x=0; x < (ssize_t) image->columns; x++)
+  {
+    if (target_info->channels == 1)
+      SetPixelGray(image,SetLCMSPixel(target_info,*p),q);
+    else
+      SetPixelRed(image,SetLCMSPixel(target_info,*p),q);
+    p++;
+    if (target_info->channels > 1)
+      {
+        SetPixelGreen(image,SetLCMSPixel(target_info,*p),q);
+        p++;
+        SetPixelBlue(image,SetLCMSPixel(target_info,*p),q);
+        p++;
+      }
+    if (target_info->channels > 3)
+      {
+        SetPixelBlack(image,SetLCMSPixel(target_info,*p),q);
+        p++;
+      }
+    q+=GetPixelChannels(image);
+  }
+}
+
+static void TransformQuantumPixels(const int id,const Image* image,
+  const LCMSInfo *source_info,const LCMSInfo *target_info,
+  const cmsHTRANSFORM *transform,Quantum *q)
+{
+  register Quantum
+    *p;
+
+  register ssize_t
+    x;
+
+  p=(Quantum *) source_info->pixels[id];
+  for (x=0; x < (ssize_t) image->columns; x++)
+  {
+    *p++=GetPixelRed(image,q);
+    if (source_info->channels > 1)
+      {
+        *p++=GetPixelGreen(image,q);
+        *p++=GetPixelBlue(image,q);
+      }
+    if (source_info->channels > 3)
+      *p++=GetLCMSPixel(source_info,GetPixelBlack(image,q));
+    q+=GetPixelChannels(image);
+  }
+  cmsDoTransform(transform[id],source_info->pixels[id],
+    target_info->pixels[id],(unsigned int) image->columns);
+  p=(Quantum *) target_info->pixels[id];
+  q-=GetPixelChannels(image)*image->columns;
+  for (x=0; x < (ssize_t) image->columns; x++)
+  {
+    if (target_info->channels == 1)
+      SetPixelGray(image,*p++,q);
+    else
+      SetPixelRed(image,*p++,q);
+    if (target_info->channels > 1)
+      {
+        SetPixelGreen(image,*p++,q);
+        SetPixelBlue(image,*p++,q);
+      }
+    if (target_info->channels > 3)
+      SetPixelBlack(image,*p++,q);
+    q+=GetPixelChannels(image);
+  }
+}
 #endif
 
 static MagickBooleanType SetsRGBImageProfile(Image *image,
@@ -815,14 +918,17 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
   const void *datum,const size_t length,ExceptionInfo *exception)
 {
 #define ProfileImageTag  "Profile/Image"
+#ifndef TYPE_XYZ_8
+  #define TYPE_XYZ_8 (COLORSPACE_SH(PT_XYZ)|CHANNELS_SH(3)|BYTES_SH(1))
+#endif
 #define ThrowProfileException(severity,tag,context) \
 { \
   if (cms_context != (cmsContext) NULL) \
     cmsDeleteContext(cms_context); \
-  if (source_profile != (cmsHPROFILE) NULL) \
-    (void) cmsCloseProfile(source_profile); \
-  if (target_profile != (cmsHPROFILE) NULL) \
-    (void) cmsCloseProfile(target_profile); \
+  if (source_info.profile != (cmsHPROFILE) NULL) \
+    (void) cmsCloseProfile(source_info.profile); \
+  if (target_info.profile != (cmsHPROFILE) NULL) \
+    (void) cmsCloseProfile(target_info.profile); \
   ThrowBinaryException(severity,tag,context); \
 }
 
@@ -884,11 +990,6 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
           value=GetImageProperty(image,"exif:InteroperabilityIndex",exception);
           if (LocaleCompare(value,"R98.") != 0)
             (void) SetsRGBImageProfile(image,exception);
-          /* Future.
-          value=GetImageProperty(image,"exif:InteroperabilityIndex",exception);
-          if (LocaleCompare(value,"R03.") != 0)
-            (void) SetAdobeRGB1998ImageProfile(image,exception);
-          */
           icc_profile=GetImageProfile(image,"icc");
         }
       if ((icc_profile != (const StringInfo *) NULL) &&
@@ -903,15 +1004,16 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
         "'%s' (LCMS)",image->filename);
 #else
       {
-        cmsHPROFILE
-          source_profile;
-
         cmsContext
           cms_context;
 
         CMSExceptionInfo
           cms_exception;
 
+        LCMSInfo
+          source_info,
+          target_info;
+
         /*
           Transform pixel colors as defined by the color profiles.
         */
@@ -922,16 +1024,16 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
           ThrowBinaryException(ResourceLimitError,
             "ColorspaceColorProfileMismatch",name);
         cmsSetLogErrorHandlerTHR(cms_context,CMSExceptionHandler);
-        source_profile=cmsOpenProfileFromMemTHR(cms_context,
+        source_info.profile=cmsOpenProfileFromMemTHR(cms_context,
           GetStringInfoDatum(profile),(cmsUInt32Number)
           GetStringInfoLength(profile));
-        if (source_profile == (cmsHPROFILE) NULL)
+        if (source_info.profile == (cmsHPROFILE) NULL)
           {
             cmsDeleteContext(cms_context);
             ThrowBinaryException(ResourceLimitError,
               "ColorspaceColorProfileMismatch",name);
           }
-        if ((cmsGetDeviceClass(source_profile) != cmsSigLinkClass) &&
+        if ((cmsGetDeviceClass(source_info.profile) != cmsSigLinkClass) &&
             (icc_profile == (StringInfo *) NULL))
           status=SetImageProfile(image,name,profile,exception);
         else
@@ -939,268 +1041,303 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             CacheView
               *image_view;
 
-            ColorspaceType
-              source_colorspace,
-              target_colorspace;
-
             cmsColorSpaceSignature
               signature;
 
-            cmsHPROFILE
-              target_profile;
-
             cmsHTRANSFORM
               *magick_restrict transform;
 
             cmsUInt32Number
-              flags,
-              source_type,
-              target_type;
+              flags;
 
-            int
-              intent;
-
-            LCMSType
-              **magick_restrict source_pixels,
-              **magick_restrict target_pixels;
-
-#if defined(LCMSHDRI)
-            LCMSType
-              source_scale,
-              target_scale;
+#if !defined(MAGICKCORE_HDRI_SUPPORT)
+            const char
+              *artifact;
 #endif
 
+            MagickBooleanType
+              highres;
+
             MagickOffsetType
               progress;
 
-            size_t
-              source_channels,
-              target_channels;
-
             ssize_t
               y;
 
-            target_profile=(cmsHPROFILE) NULL;
+            target_info.profile=(cmsHPROFILE) NULL;
             if (icc_profile != (StringInfo *) NULL)
               {
-                target_profile=source_profile;
-                source_profile=cmsOpenProfileFromMemTHR(cms_context,
+                target_info.profile=source_info.profile;
+                source_info.profile=cmsOpenProfileFromMemTHR(cms_context,
                   GetStringInfoDatum(icc_profile),
                   (cmsUInt32Number) GetStringInfoLength(icc_profile));
-                if (source_profile == (cmsHPROFILE) NULL)
+                if (source_info.profile == (cmsHPROFILE) NULL)
                   ThrowProfileException(ResourceLimitError,
                     "ColorspaceColorProfileMismatch",name);
               }
-#if defined(LCMSHDRI)
-            source_scale=1.0;
+            highres=MagickTrue;
+#if !defined(MAGICKCORE_HDRI_SUPPORT)
+            artifact=GetImageArtifact(image,"profile:highres-transform");
+            if (IsStringFalse(artifact) != MagickFalse)
+              highres=MagickFalse;
 #endif
-            source_colorspace=sRGBColorspace;
-            source_channels=3;
-            switch (cmsGetColorSpace(source_profile))
+            source_info.scale=1.0;
+            source_info.translate=0.0;
+            source_info.colorspace=sRGBColorspace;
+            source_info.channels=3;
+            switch (cmsGetColorSpace(source_info.profile))
             {
               case cmsSigCmykData:
               {
-                source_colorspace=CMYKColorspace;
-                source_channels=4;
-#if defined(LCMSHDRI)
-                source_type=(cmsUInt32Number) TYPE_CMYK_DBL;
-                source_scale=100.0;
-#else
-                source_type=(cmsUInt32Number) TYPE_CMYK_16;
+                source_info.colorspace=CMYKColorspace;
+                source_info.channels=4;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_CMYK_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_CMYK_16;
+                else
 #endif
+                  {
+                    source_info.type=(cmsUInt32Number) TYPE_CMYK_DBL;
+                    source_info.scale=100.0;
+                  }
                 break;
               }
               case cmsSigGrayData:
               {
-                source_colorspace=GRAYColorspace;
-                source_channels=1;
-#if defined(LCMSHDRI)
-                source_type=(cmsUInt32Number) TYPE_GRAY_DBL;
-#else
-                source_type=(cmsUInt32Number) TYPE_GRAY_16;
+                source_info.colorspace=GRAYColorspace;
+                source_info.channels=1;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_GRAY_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_GRAY_16;
+                else
 #endif
+                  source_info.type=(cmsUInt32Number) TYPE_GRAY_DBL;
                 break;
               }
               case cmsSigLabData:
               {
-                source_colorspace=LabColorspace;
-#if defined(LCMSHDRI)
-                source_type=(cmsUInt32Number) TYPE_Lab_DBL;
-                source_scale=100.0;
-#else
-                source_type=(cmsUInt32Number) TYPE_Lab_16;
+                source_info.colorspace=LabColorspace;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_Lab_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_Lab_16;
+                else
 #endif
+                  {
+                    source_info.type=(cmsUInt32Number) TYPE_Lab_DBL;
+                    source_info.scale=100.0;
+                    source_info.translate=(-0.5);
+                  }
                 break;
               }
-#if !defined(LCMSHDRI)
-              case cmsSigLuvData:
-              {
-                source_colorspace=YUVColorspace;
-                source_type=(cmsUInt32Number) TYPE_YUV_16;
-                break;
-              }
-#endif
               case cmsSigRgbData:
               {
-                source_colorspace=sRGBColorspace;
-#if defined(LCMSHDRI)
-                source_type=(cmsUInt32Number) TYPE_RGB_DBL;
-#else
-                source_type=(cmsUInt32Number) TYPE_RGB_16;
+                source_info.colorspace=sRGBColorspace;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_RGB_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_RGB_16;
+                else
 #endif
+                  source_info.type=(cmsUInt32Number) TYPE_RGB_DBL;
                 break;
               }
               case cmsSigXYZData:
               {
-                source_colorspace=XYZColorspace;
-#if defined(LCMSHDRI)
-                source_type=(cmsUInt32Number) TYPE_XYZ_DBL;
-#else
-                source_type=(cmsUInt32Number) TYPE_XYZ_16;
+                source_info.colorspace=XYZColorspace;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_XYZ_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_XYZ_16;
+                else
 #endif
+                  source_info.type=(cmsUInt32Number) TYPE_XYZ_DBL;
                 break;
               }
-#if !defined(LCMSHDRI)
-              case cmsSigYCbCrData:
-              {
-                source_colorspace=YUVColorspace;
-                source_type=(cmsUInt32Number) TYPE_YCbCr_16;
-                break;
-              }
-#endif
               default:
                 ThrowProfileException(ImageError,
                   "ColorspaceColorProfileMismatch",name);
             }
-            (void) source_colorspace;
-            signature=cmsGetPCS(source_profile);
-            if (target_profile != (cmsHPROFILE) NULL)
-              signature=cmsGetColorSpace(target_profile);
-#if defined(LCMSHDRI)
-            target_scale=1.0;
-#endif
-            target_channels=3;
+            signature=cmsGetPCS(source_info.profile);
+            if (target_info.profile != (cmsHPROFILE) NULL)
+              signature=cmsGetColorSpace(target_info.profile);
+            target_info.scale=1.0;
+            target_info.translate=0.0;
+            target_info.channels=3;
             switch (signature)
             {
               case cmsSigCmykData:
               {
-                target_colorspace=CMYKColorspace;
-                target_channels=4;
-#if defined(LCMSHDRI)
-                target_type=(cmsUInt32Number) TYPE_CMYK_DBL;
-                target_scale=0.01;
-#else
-                target_type=(cmsUInt32Number) TYPE_CMYK_16;
+                target_info.colorspace=CMYKColorspace;
+                target_info.channels=4;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  target_info.type=(cmsUInt32Number) TYPE_CMYK_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  target_info.type=(cmsUInt32Number) TYPE_CMYK_16;
+                else
 #endif
+                  {
+                    target_info.type=(cmsUInt32Number) TYPE_CMYK_DBL;
+                    target_info.scale=0.01;
+                  }
                 break;
               }
               case cmsSigGrayData:
               {
-                target_colorspace=GRAYColorspace;
-                target_channels=1;
-#if defined(LCMSHDRI)
-                target_type=(cmsUInt32Number) TYPE_GRAY_DBL;
-#else
-                target_type=(cmsUInt32Number) TYPE_GRAY_16;
+                target_info.colorspace=GRAYColorspace;
+                target_info.channels=1;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  target_info.type=(cmsUInt32Number) TYPE_GRAY_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  target_info.type=(cmsUInt32Number) TYPE_GRAY_16;
+                else
 #endif
+                  target_info.type=(cmsUInt32Number) TYPE_GRAY_DBL;
                 break;
               }
               case cmsSigLabData:
               {
-                target_colorspace=LabColorspace;
-#if defined(LCMSHDRI)
-                target_type=(cmsUInt32Number) TYPE_Lab_DBL;
-                target_scale=0.01;
-#else
-                target_type=(cmsUInt32Number) TYPE_Lab_16;
+                target_info.colorspace=LabColorspace;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  target_info.type=(cmsUInt32Number) TYPE_Lab_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  target_info.type=(cmsUInt32Number) TYPE_Lab_16;
+                else
 #endif
+                  {
+                    target_info.type=(cmsUInt32Number) TYPE_Lab_DBL;
+                    target_info.scale=0.01;
+                    target_info.translate=0.5;
+                  }
                 break;
               }
-#if !defined(LCMSHDRI)
-              case cmsSigLuvData:
-              {
-                target_colorspace=YUVColorspace;
-                target_type=(cmsUInt32Number) TYPE_YUV_16;
-                break;
-              }
-#endif
               case cmsSigRgbData:
               {
-                target_colorspace=sRGBColorspace;
-#if defined(LCMSHDRI)
-                target_type=(cmsUInt32Number) TYPE_RGB_DBL;
-#else
-                target_type=(cmsUInt32Number) TYPE_RGB_16;
+                target_info.colorspace=sRGBColorspace;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  target_info.type=(cmsUInt32Number) TYPE_RGB_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  target_info.type=(cmsUInt32Number) TYPE_RGB_16;
+                else
 #endif
+                  target_info.type=(cmsUInt32Number) TYPE_RGB_DBL;
                 break;
               }
               case cmsSigXYZData:
               {
-                target_colorspace=XYZColorspace;
-#if defined(LCMSHDRI)
-                target_type=(cmsUInt32Number) TYPE_XYZ_DBL;
-#else
-                target_type=(cmsUInt32Number) TYPE_XYZ_16;
+                target_info.colorspace=XYZColorspace;
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
+                if (highres == MagickFalse)
+                  target_info.type=(cmsUInt32Number) TYPE_XYZ_8;
+                else
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+                if (highres == MagickFalse)
+                  source_info.type=(cmsUInt32Number) TYPE_XYZ_16;
+                else
 #endif
+                  target_info.type=(cmsUInt32Number) TYPE_XYZ_DBL;
                 break;
               }
-#if !defined(LCMSHDRI)
-              case cmsSigYCbCrData:
-              {
-                target_colorspace=YUVColorspace;
-                target_type=(cmsUInt32Number) TYPE_YCbCr_16;
-                break;
-              }
-#endif
               default:
                 ThrowProfileException(ImageError,
                   "ColorspaceColorProfileMismatch",name);
             }
             switch (image->rendering_intent)
             {
-              case AbsoluteIntent: intent=INTENT_ABSOLUTE_COLORIMETRIC; break;
-              case PerceptualIntent: intent=INTENT_PERCEPTUAL; break;
-              case RelativeIntent: intent=INTENT_RELATIVE_COLORIMETRIC; break;
-              case SaturationIntent: intent=INTENT_SATURATION; break;
-              default: intent=INTENT_PERCEPTUAL; break;
+              case AbsoluteIntent:
+              {
+                target_info.intent=INTENT_ABSOLUTE_COLORIMETRIC;
+                break;
+              }
+              case PerceptualIntent:
+              {
+                target_info.intent=INTENT_PERCEPTUAL;
+                break;
+              }
+              case RelativeIntent:
+              {
+                target_info.intent=INTENT_RELATIVE_COLORIMETRIC;
+                break;
+              }
+              case SaturationIntent:
+              {
+                target_info.intent=INTENT_SATURATION;
+                break;
+              }
+              default:
+              {
+                target_info.intent=INTENT_PERCEPTUAL;
+                break;
+              }
             }
             flags=cmsFLAGS_HIGHRESPRECALC;
 #if defined(cmsFLAGS_BLACKPOINTCOMPENSATION)
             if (image->black_point_compensation != MagickFalse)
               flags|=cmsFLAGS_BLACKPOINTCOMPENSATION;
 #endif
-            transform=AcquireTransformThreadSet(source_profile,source_type,
-              target_profile,target_type,intent,flags,cms_context);
+            transform=AcquireTransformThreadSet(&source_info,&target_info,
+              flags,cms_context);
             if (transform == (cmsHTRANSFORM *) NULL)
               ThrowProfileException(ImageError,"UnableToCreateColorTransform",
                 name);
             /*
               Transform image as dictated by the source & target image profiles.
             */
-            source_pixels=AcquirePixelThreadSet(image->columns,source_channels);
-            target_pixels=AcquirePixelThreadSet(image->columns,target_channels);
-            if ((source_pixels == (LCMSType **) NULL) ||
-                (target_pixels == (LCMSType **) NULL))
+            source_info.pixels=AcquirePixelThreadSet(image->columns,
+              source_info.channels,highres);
+            target_info.pixels=AcquirePixelThreadSet(image->columns,
+              target_info.channels,highres);
+            if ((source_info.pixels == (void **) NULL) ||
+                (target_info.pixels == (void **) NULL))
               {
-                target_pixels=DestroyPixelThreadSet(target_pixels);
-                source_pixels=DestroyPixelThreadSet(source_pixels);
+                target_info.pixels=DestroyPixelThreadSet(target_info.pixels);
+                source_info.pixels=DestroyPixelThreadSet(source_info.pixels);
                 transform=DestroyTransformThreadSet(transform);
                 ThrowProfileException(ResourceLimitError,
                   "MemoryAllocationFailed",image->filename);
               }
             if (SetImageStorageClass(image,DirectClass,exception) == MagickFalse)
               {
-                target_pixels=DestroyPixelThreadSet(target_pixels);
-                source_pixels=DestroyPixelThreadSet(source_pixels);
+                target_info.pixels=DestroyPixelThreadSet(target_info.pixels);
+                source_info.pixels=DestroyPixelThreadSet(source_info.pixels);
                 transform=DestroyTransformThreadSet(transform);
-                if (source_profile != (cmsHPROFILE) NULL)
-                  (void) cmsCloseProfile(source_profile);
-                if (target_profile != (cmsHPROFILE) NULL)
-                  (void) cmsCloseProfile(target_profile);
+                if (source_info.profile != (cmsHPROFILE) NULL)
+                  (void) cmsCloseProfile(source_info.profile);
+                if (target_info.profile != (cmsHPROFILE) NULL)
+                  (void) cmsCloseProfile(target_info.profile);
                 return(MagickFalse);
               }
-            if (target_colorspace == CMYKColorspace)
-              (void) SetImageColorspace(image,target_colorspace,exception);
+            if (target_info.colorspace == CMYKColorspace)
+              (void) SetImageColorspace(image,target_info.colorspace,exception);
             progress=0;
             image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
@@ -1215,15 +1352,9 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               MagickBooleanType
                 sync;
 
-              register LCMSType
-                *p;
-
               register Quantum
                 *magick_restrict q;
 
-              register ssize_t
-                x;
-
               if (status == MagickFalse)
                 continue;
               q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
@@ -1233,44 +1364,10 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                   status=MagickFalse;
                   continue;
                 }
-              p=source_pixels[id];
-              for (x=0; x < (ssize_t) image->columns; x++)
-              {
-                *p++=LCMSScaleSource(GetPixelRed(image,q));
-                if (source_channels > 1)
-                  {
-                    *p++=LCMSScaleSource(GetPixelGreen(image,q));
-                    *p++=LCMSScaleSource(GetPixelBlue(image,q));
-                  }
-                if (source_channels > 3)
-                  *p++=LCMSScaleSource(GetPixelBlack(image,q));
-                q+=GetPixelChannels(image);
-              }
-              cmsDoTransform(transform[id],source_pixels[id],target_pixels[id],
-                (unsigned int) image->columns);
-              p=target_pixels[id];
-              q-=GetPixelChannels(image)*image->columns;
-              for (x=0; x < (ssize_t) image->columns; x++)
-              {
-                if (target_channels == 1)
-                  SetPixelGray(image,LCMSScaleTarget(*p),q);
-                else
-                  SetPixelRed(image,LCMSScaleTarget(*p),q);
-                p++;
-                if (target_channels > 1)
-                  {
-                    SetPixelGreen(image,LCMSScaleTarget(*p),q);
-                    p++;
-                    SetPixelBlue(image,LCMSScaleTarget(*p),q);
-                    p++;
-                  }
-                if (target_channels > 3)
-                  {
-                    SetPixelBlack(image,LCMSScaleTarget(*p),q);
-                    p++;
-                  }
-                q+=GetPixelChannels(image);
-              }
+              if (highres != MagickFalse)
+                TransformDoublePixels(id,image,&source_info,&target_info,transform,q);
+              else
+                TransformQuantumPixels(id,image,&source_info,&target_info,transform,q);
               sync=SyncCacheViewAuthenticPixels(image_view,exception);
               if (sync == MagickFalse)
                 status=MagickFalse;
@@ -1290,7 +1387,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                 }
             }
             image_view=DestroyCacheView(image_view);
-            (void) SetImageColorspace(image,target_colorspace,exception);
+            (void) SetImageColorspace(image,target_info.colorspace,exception);
             switch (signature)
             {
               case cmsSigRgbData:
@@ -1314,16 +1411,16 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               default:
                 break;
             }
-            target_pixels=DestroyPixelThreadSet(target_pixels);
-            source_pixels=DestroyPixelThreadSet(source_pixels);
+            target_info.pixels=DestroyPixelThreadSet(target_info.pixels);
+            source_info.pixels=DestroyPixelThreadSet(source_info.pixels);
             transform=DestroyTransformThreadSet(transform);
             if ((status != MagickFalse) &&
-                (cmsGetDeviceClass(source_profile) != cmsSigLinkClass))
+                (cmsGetDeviceClass(source_info.profile) != cmsSigLinkClass))
               status=SetImageProfile(image,name,profile,exception);
-            if (target_profile != (cmsHPROFILE) NULL)
-              (void) cmsCloseProfile(target_profile);
+            if (target_info.profile != (cmsHPROFILE) NULL)
+              (void) cmsCloseProfile(target_info.profile);
           }
-        (void) cmsCloseProfile(source_profile);
+        (void) cmsCloseProfile(source_info.profile);
         cmsDeleteContext(cms_context);
       }
 #endif
@@ -1761,13 +1858,7 @@ static MagickBooleanType ValidateXMPProfile(const StringInfo *profile)
 #else
 static MagickBooleanType ValidateXMPProfile(const StringInfo *profile)
 {
-  char
-    *p;
-
-  p=StringLocateSubstring((const char *) GetStringInfoDatum(profile),"x:xmpmeta");
-  if (p != (char *) NULL)
-    p=StringLocateSubstring((const char *) GetStringInfoDatum(profile),"rdf:RDF");
-  return(p == (char *) NULL ? MagickFalse : MagickTrue);
+  return(MagickFalse);
 }
 #endif
 
@@ -1776,8 +1867,7 @@ static MagickBooleanType SetImageProfileInternal(Image *image,const char *name,
   ExceptionInfo *exception)
 {
   char
-    key[MagickPathExtent],
-    property[MagickPathExtent];
+    key[MagickPathExtent];
 
   MagickBooleanType
     status;
@@ -1808,11 +1898,6 @@ static MagickBooleanType SetImageProfileInternal(Image *image,const char *name,
         if (recursive == MagickFalse)
           WriteTo8BimProfile(image,name,profile);
     }
-  /*
-    Inject profile into image properties.
-  */
-  (void) FormatLocaleString(property,MagickPathExtent,"%s:*",name);
-  (void) GetImageProperty(image,property,exception);
   return(status);
 }
 
